@@ -390,8 +390,12 @@ soup_gnutls_pull_func (gnutls_transport_ptr_t transport_data,
 	SoupGNUTLSChannel *chan = transport_data;
 	ssize_t nread;
 
-	nread = read (chan->sockfd, buf, buflen);
+	nread = recv (chan->sockfd, buf, buflen, 0);
+#ifdef G_OS_WIN32
+	chan->eagain = (nread == SOCKET_ERROR && WSAGetLastError () == WSAEWOULDBLOCK);
+#else
 	chan->eagain = (nread == -1 && errno == EAGAIN);
+#endif
 	return nread;
 }
 
@@ -402,8 +406,12 @@ soup_gnutls_push_func (gnutls_transport_ptr_t transport_data,
 	SoupGNUTLSChannel *chan = transport_data;
 	ssize_t nwrote;
 
-	nwrote = write (chan->sockfd, buf, buflen);
+	nwrote = send (chan->sockfd, buf, buflen, 0);
+#ifdef G_OS_WIN32
+	chan->eagain = (nwrote == SOCKET_ERROR && WSAGetLastError () == WSAEWOULDBLOCK);
+#else
 	chan->eagain = (nwrote == -1 && errno == EAGAIN);
+#endif
 	return nwrote;
 }
 
@@ -446,7 +454,8 @@ soup_ssl_wrap_iochannel (GIOChannel *sock, gboolean non_blocking,
 	if (ret)
 		goto THROW_CREATE_ERROR;
 
-	if (gnutls_set_default_priority (session) != 0)
+	/* See http://bugzilla.gnome.org/show_bug.cgi?id=581342 */
+	if (gnutls_priority_set_direct (session, "NORMAL:!VERS-TLS1.1:!VERS-TLS1.0", NULL) != 0)
 		goto THROW_CREATE_ERROR;
 
 	if (gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE,
@@ -528,6 +537,10 @@ soup_ssl_get_client_credentials (const char *ca_file)
 	creds = g_slice_new0 (SoupSSLCredentials);
 	gnutls_certificate_allocate_credentials (&creds->creds);
 
+	/* http://bugzilla.gnome.org/show_bug.cgi?id=589323 */
+	gnutls_certificate_set_verify_flags (creds->creds,
+					     GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
+
 	if (ca_file) {
 		creds->have_ca_file = TRUE;
 		status = gnutls_certificate_set_x509_trust_file (
