@@ -33,6 +33,8 @@ guint              soup_str_case_hash        (gconstpointer key);
 gboolean           soup_str_case_equal       (gconstpointer v1,
 					      gconstpointer v2);
 
+#define _SOUP_ATOMIC_INTERN_STRING(variable, value) (g_once_init_enter ((gsize *)&variable) ? (g_once_init_leave ((gsize *)&variable, GPOINTER_TO_SIZE (g_intern_static_string (value))), variable) : variable)
+
 /* SSL stuff */
 
 extern const gboolean soup_ssl_supported;
@@ -44,7 +46,7 @@ GQuark soup_ssl_error_quark (void);
 typedef enum {
 	SOUP_SSL_ERROR_HANDSHAKE_NEEDS_READ,
 	SOUP_SSL_ERROR_HANDSHAKE_NEEDS_WRITE,
-	SOUP_SSL_ERROR_CERTIFICATE,
+	SOUP_SSL_ERROR_CERTIFICATE
 } SoupSSLError;
 
 G_END_DECLS
