@@ -965,6 +965,8 @@ soup_uri_host_hash (gconstpointer key)
 {
 	const SoupURI *uri = key;
 
+	g_return_val_if_fail (uri != NULL && uri->host != NULL, 0);
+
 	return GPOINTER_TO_UINT (uri->scheme) + uri->port +
 		soup_str_case_hash (uri->host);
 }
@@ -987,6 +989,9 @@ soup_uri_host_equal (gconstpointer v1, gconstpointer v2)
 	const SoupURI *one = v1;
 	const SoupURI *two = v2;
 
+	g_return_val_if_fail (one != NULL && two != NULL, one == two);
+	g_return_val_if_fail (one->host != NULL && two->host != NULL, one->host == two->host);
+
 	if (one->scheme != two->scheme)
 		return FALSE;
 	if (one->port != two->port)
