@@ -21,6 +21,7 @@ static void test_server_shutdown (void);
 static SoupLogger *logger;
 
 int debug_level, errors;
+gboolean expect_warning;
 static int http_debug_level;
 
 static gboolean
@@ -66,9 +67,15 @@ static void
 test_log_handler (const char *log_domain, GLogLevelFlags log_level,
 		  const char *message, gpointer user_data)
 {
+	if (log_level & (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL)) {
+		if (expect_warning) {
+			expect_warning = FALSE;
+			debug_printf (2, "Got expected warning: %s\n", message);
+			return;
+		} else
+			errors++;
+	}
 	g_log_default_handler (log_domain, log_level, message, user_data);
-	if (log_level & (G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL))
-		errors++;
 }
 
 void
@@ -253,13 +260,26 @@ soup_test_session_abort_unref (SoupSession *session)
 
 static gpointer run_server_thread (gpointer user_data);
 
-SoupServer *
-soup_test_server_new (gboolean in_own_thread)
+static SoupServer *
+test_server_new (gboolean in_own_thread, gboolean ssl)
 {
 	GMainContext *async_context;
+	const char *ssl_cert_file, *ssl_key_file;
+
+	if (test_server)
+		test_server_shutdown ();
 
 	async_context = in_own_thread ? g_main_context_new () : NULL;
+
+	if (ssl) {
+		ssl_cert_file = SRCDIR "/test-cert.pem";
+		ssl_key_file = SRCDIR "/test-key.pem";
+	} else
+		ssl_cert_file = ssl_key_file = NULL;
+
 	test_server = soup_server_new (SOUP_SERVER_ASYNC_CONTEXT, async_context,
+				       SOUP_SERVER_SSL_CERT_FILE, ssl_cert_file,
+				       SOUP_SERVER_SSL_KEY_FILE, ssl_key_file,
 				       NULL);
 	if (async_context)
 		g_main_context_unref (async_context);
@@ -278,6 +298,18 @@ soup_test_server_new (gboolean in_own_thread)
 	return test_server;
 }
 
+SoupServer *
+soup_test_server_new (gboolean in_own_thread)
+{
+	return test_server_new (in_own_thread, FALSE);
+}
+
+SoupServer *
+soup_test_server_new_ssl (gboolean in_own_thread)
+{
+	return test_server_new (in_own_thread, TRUE);
+}
+
 static gpointer
 run_server_thread (gpointer user_data)
 {
